<?php

namespace App\Services\File\Upload;

use Illuminate\Http\Request;

class FilesUploadService
{
    use FilesUploadTrait;

    protected int $filesSize = 0;

    protected ?ProgressBar $progressBar = null;

    public function __construct(protected Request $request)
    {
        $this->filesSize = $this->checkRequestData($this->request);

        if ($this->request->get("request_id")) {
            $this->progressBar = new ProgressBar($this->request->get("request_id"), $this->filesSize);
        }
    }

    public function upload(): array
    {
        $items = [];
        $fileNames = $this->request->get("file_names", []);
        foreach ($this->request->file("files") as $index => $file) {

            $item = $this->uploadFile($this->request, $file, $this->progressBar);
            $item['file']['title'] = $fileNames[$index] ?? $item['file']['title'];

            $items[] = $item;
        }

        $uploadedFiles = $this->storeFiles($items);

        $this->updateUsedStorageCapacity($this->filesSize);

        $this->progressBar?->completed();

        return $uploadedFiles;
    }
}
